/* SPDX-License-Identifier: GPL-2.0 WITH Linux-syscall-note */
/*
 * Platform Firmware Runtime Update header
 *
 * Copyright(c) 2021 Intel Corporation. All rights reserved.
 */
#ifndef __PFRU_H__
#define __PFRU_H__

#include <linux/ioctl.h>
#include <linux/uuid.h>

#define PFRU_UUID		"ECF9533B-4A3C-4E89-939E-C77112601C6D"
#define PFRU_CODE_INJ_UUID		"B2F84B79-7B6E-4E45-885F-3FB9BB185402"
#define PFRU_DRV_UPDATE_UUID		"4569DD8C-75F1-429A-A3D6-24DE8097A0DF"

#define FUNC_STANDARD_QUERY	0
#define FUNC_QUERY_UPDATE_CAP	1
#define FUNC_QUERY_BUF		2
#define FUNC_START		3

#define CODE_INJECT_TYPE	1
#define DRIVER_UPDATE_TYPE	2

#define REVID_1		1
#define REVID_2		2

#define PFRU_MAGIC 0xEE

#define PFRU_IOC_SET_REV _IOW(PFRU_MAGIC, 0x01, unsigned int)
#define PFRU_IOC_STAGE _IOW(PFRU_MAGIC, 0x02, unsigned int)
#define PFRU_IOC_ACTIVATE _IOW(PFRU_MAGIC, 0x03, unsigned int)
#define PFRU_IOC_STAGE_ACTIVATE _IOW(PFRU_MAGIC, 0x04, unsigned int)

static inline int valid_revid(int id)
{
	return (id == REVID_1) || (id == REVID_2);
}

/* Capsule file payload header */
struct payload_hdr {
	__u32	sig;
	__u32	hdr_version;
	__u32	hdr_size;
	__u32	hw_ver;
	__u32	rt_ver;
	guid_t	platform_id;
};

enum start_action {
	START_STAGE,
	START_ACTIVATE,
	START_STAGE_ACTIVATE,
};

enum dsm_status {
	DSM_SUCCEED,
	DSM_FUNC_NOT_SUPPORT,
	DSM_INVAL_INPUT,
	DSM_HARDWARE_ERR,
	DSM_RETRY_SUGGESTED,
	DSM_UNKNOWN,
	DSM_FUNC_SPEC_ERR,
};

struct update_cap_info {
	enum dsm_status status;
	int update_cap;

	guid_t code_type;
	int fw_version;
	int code_rt_version;

	guid_t drv_type;
	int drv_rt_version;
	int drv_svn;

	guid_t platform_id;
	guid_t oem_id;

	char oem_info[];
};

struct com_buf_info {
	enum dsm_status status;
	enum dsm_status ext_status;
	unsigned long addr_lo;
	unsigned long addr_hi;
	int buf_size;
};

struct capsulate_buf_info {
	unsigned long src;
	int size;
};

struct updated_result {
	enum dsm_status status;
	enum dsm_status ext_status;
	unsigned long low_auth_time;
	unsigned long high_auth_time;
	unsigned long low_exec_time;
	unsigned long high_exec_time;
};

#define PFRU_TELEMETRY_UUID	"75191659-8178-4D9D-B88F-AC5E5E93E8BF"

/* Telemetry structures. */
struct telem_data_info {
	enum dsm_status status;
	enum dsm_status ext_status;
	/* Maximum supported size of data of
	 * all Data Chunks combined.
	 */
	unsigned long chunk1_addr_lo;
	unsigned long chunk1_addr_hi;
	unsigned long chunk2_addr_lo;
	unsigned long chunk2_addr_hi;
	int max_data_size;
	int chunk1_size;
	int chunk2_size;
	int rollover_cnt;
	int reset_cnt;
};

struct telem_info {
	int log_level;
	int log_type;
	int log_revid;
};

/* Two logs: history and execution log */
#define LOG_EXEC_IDX	0
#define LOG_HISTORY_IDX	1
#define NR_LOG_TYPE	2

#define LOG_ERR		0
#define LOG_WARN	1
#define LOG_INFO	2
#define LOG_VERB	4

#define FUNC_SET_LEV		1
#define FUNC_GET_LEV		2
#define FUNC_GET_DATA		3

#define LOG_NAME_SIZE		10

#define PFRU_LOG_IOC_SET_INFO _IOW(PFRU_MAGIC, 0x05, struct telem_info)
#define PFRU_LOG_IOC_GET_INFO _IOR(PFRU_MAGIC, 0x06, struct telem_info)
#define PFRU_LOG_IOC_GET_DATA_INFO _IOR(PFRU_MAGIC, 0x07, struct telem_data_info)

#endif /* __PFRU_H__ */
